import re
import readline

# Usage:
#
# set_parameters(list_=['suggestion', 'suggestion2', 'other_suggestion'])
# result = get_input_autocomplete()
# print('Response -> ** % s**' % result)

completion_list = ['one', 'two', 'thee']
is_case_sensitive = True


def set_parameters(list_, case_sensitive=True):
    """ Set module parameters """

    global completion_list, is_case_sensitive

    completion_list = list_
    is_case_sensitive = case_sensitive


def autocomplete(text, state):
    """ Generic readline completion entry point. """

    buffer = readline.get_line_buffer()

    comp = completion_list
    if not is_case_sensitive:
        buffer = buffer.lower()
        comp = [c.lower() for c in completion_list]

    results = [c for c in comp if c.startswith(buffer)] + [None]

    # Handle breaking characters by truncating strings at the last breaking character
    strip_pos = find_breaking_strings(buffer)
    if strip_pos > 0:
        results = [i[strip_pos + 1:]
                   for i in results if i is not None] + [None]

    return results[state]


def find_breaking_strings(string):
    """ Find last breaking string in a string """

    breaking_strings = [' ', '@', '?', '#', '$', '%', '&', '*']
    result = 0
    for breaking_string in breaking_strings:
        rf = string.rfind(breaking_string)
        if rf > result:
            result = rf

    return result


def get_input_autocomplete(message=''):
    """ Allow user to type input and provide auto-completion """

    # Apple does not ship GNU readline with OS X.
    # It does ship BSD libedit which includes a readline compatibility interface.
    # Source: https://stackoverflow.com/questions/7116038/python-tab-completion-mac-osx-10-7-lion
    if 'libedit' in readline.__doc__:
        readline.parse_and_bind("bind ^I rl_complete")
    else:
        readline.parse_and_bind("tab: complete")
    readline.set_completer(autocomplete)

    try:
        return input(message).strip()
    except KeyboardInterrupt:
        return False



from ..base import BaseTest
from ...lib.Encryption import Encryption
from ...lib.Config import Config
from ...modules.carry import global_scope


class Test(BaseTest):

    def test_enc(self):
        self.assertIsInstance(global_scope['enc'], Encryption)

    def test_db_file(self):
        self.assertIsInstance(global_scope['db_file'], str)

    def test_conf(self):
        self.assertIsInstance(global_scope['conf'], Config)



from unittest.mock import patch

import pyperclip

from ..base import BaseTest
from ...views import clipboard
from ...modules.carry import global_scope


class Test(BaseTest):

    @patch.object(pyperclip, 'copy')
    def test_copy(self, patched):
        patched.return_value = None
        clipboard.copy('some string')
        self.assertEqual(clipboard.get_signature(
            'some string'), '61d034473102d7dac305902770471fd50f4c5b26f6831a56dd90b5184b3c30fc')

    @patch.object(pyperclip, 'copy')
    def test_copy_none(self, patched):
        patched.return_value = None
        assert clipboard.copy(None) is False

    @patch.object(pyperclip, 'paste')
    def test_is_changed(self, patched):
        patched.return_value = 'some string'
        clipboard.clipboard_signature = clipboard.get_signature('some string')
        self.assertFalse(clipboard.is_changed())

    @patch.object(pyperclip, 'paste')
    def test_is_changed_2(self, patched):
        patched.return_value = 'some other string'
        clipboard.clipboard_signature = clipboard.get_signature('some string')
        self.assertTrue(clipboard.is_changed())

    def test_is_changed_3(self):
        clipboard.clipboard_signature = ''
        self.assertIsNone(clipboard.is_changed())

    def test_getSignature(self):
        self.assertEqual(clipboard.get_signature(
            'some string'), '61d034473102d7dac305902770471fd50f4c5b26f6831a56dd90b5184b3c30fc')

    @patch.object(pyperclip, 'copy')
    @patch.object(pyperclip, 'paste')
    def test_wait(self, patched, patched2):
        patched.return_value = 'some string'
        patched2.return_value = 'some string'
        clipboard.clipboard_signature = clipboard.get_signature('some string')
        # Ensure we have a short wait time
        global_scope['conf'].update('clipboardTTL', '1')

        self.assertIsNone(clipboard.wait())
        self.assertEqual(clipboard.clipboard_signature, '')

    @patch.object(pyperclip, 'copy')
    @patch.object(pyperclip, 'paste')
    def test_wait_2(self, patched, patched2):
        patched.return_value = 'some other string'
        patched2.return_value = 'some string'
        clipboard.clipboard_signature = clipboard.get_signature('some string')
        # Ensure we have a short wait time
        global_scope['conf'].update('clipboardTTL', '1')

        self.assertIsNone(clipboard.wait())
        self.assertEqual(clipboard.clipboard_signature, '')

    def test_wait_3(self):
        # Case when we have no clipboard_signature because
        # pyperclip is not available
        self.assertEqual(clipboard.clipboard_signature, '')
        self.assertIsNone(clipboard.wait())

    @patch.object(pyperclip, 'copy')
    @patch.object(pyperclip, 'paste')
    def test_erase(self, patched, patched2):
        patched.return_value = 'some string'
        patched2.return_value = 'some string'
        clipboard.erase()
        self.assertEqual(clipboard.clipboard_signature, '')



from setuptools import setup, find_packages
import pathlib

HERE = pathlib.Path(__file__).parent

try:
    README = (HERE / "README.md").read_text()
except FileNotFoundError:
    README = "Python Password Manager"

setup(
    name='2025_assignment2_vaultPasswordManager',
    version='1.0.0',  
    description='Python Password Manager',
    long_description=README,
    long_description_content_type='text/markdown', 
    author='EmacsSoftwares',
    author_email='matteo.broglio3@gmail.com',
    url='https://gitlab.com/mbroglio/2025_assignment2_vaultPasswordManager',
    license='MIT',

    package_dir={'': 'src'},
    packages=find_packages(where='src'),

    install_requires=[
        'pycryptodome==3.20.0',
        'pyperclip',
        'tabulate',
        'passwordgenerator',
        'SQLAlchemy==1.4.41',
        'sqlcipher3==0.5.4'
    ],
    entry_points={
        'console_scripts': [
            'vault = vault.vault:main',
        ],
    },
    classifiers=[
        'Topic :: Security',
        'Topic :: Security :: Cryptography',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
    ],
)


from unittest.mock import patch

from sqlalchemy import engine
from sqlalchemy.orm import Session

from ..base import BaseTest
from ...views import change_key
from ...modules.carry import global_scope
from ...lib.Encryption import Encryption
from ...models.Secret import SecretModel
from ...models.User import UserModel


class Test(BaseTest):

    def setUp(self):
        # Set instances of Encryption for the current and the new key
        change_key.enc_current = global_scope['enc']
        # Forcing same key to not invalidate the db for further tests
        change_key.enc_new = Encryption(global_scope['enc'].key)

        # Create some secrets
        secret_1 = SecretModel(name='Paypal',
                               url='https://www.paypal.com',
                               login='gab@gmail.com',
                               password='password123',
                               notes='Some notes')
        self.session.add(secret_1)
        secret_2 = SecretModel(name='Gmail',
                               url='https://www.gmail.com',
                               login='gab@gmail.com',
                               password='password;123',
                               notes='Some notes\nsome more notes')
        self.session.add(secret_2)
        secret_3 = SecretModel(name='eBay',
                               url='https://www.ebay.com',
                               login='gab@gmail.com',
                               password='123password',
                               notes='')
        self.session.add(secret_3)

        # Create validation key
        key_salt = global_scope['enc'].key + \
            global_scope['conf'].salt.encode()

        # Save user
        user = UserModel(key='key_validation',
                         value=global_scope['enc'].encrypt(key_salt))
        self.session.add(user)

        self.session.commit()

    def tearDown(self):
        # Truncate users table
        self.session.query(UserModel).delete()
        self.session.commit()

    def test_rekey(self):
        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(change_key.rekey())

    @patch.object(change_key, 'get_key_input')
    def test_rekey_2(self, patched):
        patched.return_value = False

        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertFalse(change_key.rekey())

    def test_rekey_secrets(self):
        self.assertTrue(change_key.rekey_secrets())

    def test_rekey_validation_key(self):
        self.assertTrue(change_key.rekey_validation_key())

    # def test_rekey_db(self):
    #     self.assertTrue(change_key.rekey_db())

    def test_unlock(self):
        with patch('getpass.getpass', return_value=self.secret_key):
            self.assertTrue(change_key.unlock())



import getpass
import sys
import uuid

from ..models.base import Base, get_session, get_engine
from ..models.Category import CategoryModel  # Imported for schema creation
from ..models.Secret import SecretModel  # Imported for schema creation
from ..modules.carry import global_scope
from .users import validation_key_new
from .menu import get_input
from ..lib.Encryption import Encryption


def initialize(salt):
    """
        Vault setup
    """

    print('Welcome to Vault. Please choose a secure secret key.')
    print()

    while True:
        # Ask user for a master key
        key = get_key_input()
        if key is False:
            return False

        if key:
            # Create Encryption instance and set it to the global scope
            global_scope['enc'] = Encryption(key.encode())

            # Create db
            create_db()

            # Create validation key
            validation_key_new()

            print()
            print("Your vault has been created and encrypted with your master key.")
            print("Your unique salt is: %s " % (salt))
            print(
                "Write it down. If you lose your config file you will need it to unlock your vault.")

            return True


def create_db():
    """
        Create db
    """

    session = get_session()
    Base.metadata.create_all(get_engine())
    session.commit()

    return True


def get_key_input():
    """
        Prompt user for a master key
    """

    key = get_input(message='Please choose a master key:',
                    secure=True, check_timer=False)
    if key is False:
        return False
    repeat = get_input(
        message='Please confirm your master key:', secure=True, check_timer=False)
    if repeat is False:
        return False

    # Ensure that the key was correctly typed twice and is valid
    if check_key_and_repeat(key, repeat) and is_key_valid(key):
        return key

    return None


def is_key_valid(key):
    """
        Return True if a key is valid
    """

    if len(key) < 8:
        print()
        print('The master key should be at least 8 characters. Please try again!')
        print()

        return False

    return True


def check_key_and_repeat(key, repeat):
    """
        Ensure that the key was correctly typed twice
    """

    if key != repeat:
        print()
        print('The master key does not match its confirmation. Please try again!')
        print()

        return False

    return True



import sys
import os


def logo():
    """
        No comment.
    """

    print(r"    __      __         _ _           .----.    ")
    print(r"    \ \    / /        | | |         / /  \ \   ")
    print(r"     \ \  / /_ _ _   _| | |_       _| |__| |_  ")
    print(r"      \ \/ / _` | | | | | __|    .' |_   |_| '.")
    print(r"       \  / (_| | |_| | | |_     '.__________.'")
    print(r"        \/ \__,_|\__,_|_|\__|    |            |")
    print(r"                                 '.__________.'")


def logo_small():
    """
        No comment.
    """

    print(r"                  _ _   ")
    print(r"__   ____ _ _   _| | |_ ")
    print(r"\ \ / / _` | | | | | __|")
    print(r" \ V / (_| | |_| | | |_ ")
    print(r"  \_/ \__,_|\__,_|_|\__|")


def create_directory_if_missing(dir_):
    """
        Create the vault and configuration file storage folder if it does not exist
    """

    import os

    try:
        if not os.path.exists(dir_):
            os.makedirs(dir_)

            return True

        return False
    except Exception:
        import sys

        print()
        print('We were unable to create the folder `%s` to store the vault and configuration file.' % (
            dir_))
        print('Please check the permissions or run `./vault.py --help` to find out how to specify an alternative path for both files.')
        print()
        sys.exit()


def assess_integrity(vault_path, config_path):
    """
        The vault config file contains a salt. The salt is used to unlock the vault along with the master key.
        By default, config files are created automatically. A new config file will not allow to open an existing vault.
        We are ensuring here that a config file exists if a vault exists.
    """

    import os
    import sys

    if not os.path.isfile(config_path) and os.path.isfile(vault_path):
        print()
        print("It looks like you have a vault setup but your config file is missing.")
        print("The vault cannot be unlocked without a critical piece of information from the config file (the salt).")
        print("Please restore the config file before proceeding.")
        print()
        sys.exit()


def erase_vault(vault_path, config_path):
    """
        Will erase the vault and config file after asking user for confirmation
    """

    import os
    import sys

    print()
    if confirm(prompt='Do you want to permanently erase your vault? All your data will be lost!', resp=False):
        # Delete files
        if os.path.isfile(vault_path):
            os.remove(vault_path)
        if os.path.isfile(config_path):
            os.remove(config_path)

        print()
        print('The vault and config file have been deleted.')
        print()
        sys.exit()
    else:
        sys.exit()


def confirm(prompt=None, resp=False):
    """
        Source: http://code.activestate.com/recipes/541096-prompt-the-user-for-confirmation/

        prompts for yes or no response from the user. Returns True for yes and
        False for no.

        'resp' should be set to the default value assumed by the caller when
        user simply types ENTER.

        >>> confirm(prompt='Create Directory?', resp=True)
        Create Directory? [y]|n:
        True
    """

    if prompt is None:
        prompt = 'Confirm'

    if resp:
        prompt = '%s [Y/n]: ' % (prompt)
    else:
        prompt = '%s [y/N]: ' % (prompt)

    while True:
        ans = input(prompt)
        if not ans:
            return resp
        if ans not in ['y', 'Y', 'n', 'N']:
            print('please enter y or n.')
            continue
        if ans == 'y' or ans == 'Y':
            return True
        if ans == 'n' or ans == 'N':
            return False


def is_unicode_supported():
    """
        Returns `True` if stdout supports unicode
    """

    if sys.stdout.encoding:
        return sys.stdout.encoding.lower().startswith('utf-')

    return False


def lock_prefix():
    """
        Will prefix locks with a Unicode Character 'KEY' (U+1F511)
        if the user's stdout supports it
    """

    if is_unicode_supported():
        return u'\U0001F511  '  # Extra spaces are intentional

    return ''


def clear_screen():
    """
        Clear user window
    """

    print('\x1b[1J')

    return True
